@@ -9,10 +9,11 @@ from torch import Tensor
 from torch.utils.data import Dataset
 from torch_geometric.utils import subgraph
 
-from tsl.typing import TensArray, TemporalIndex, IndexSlice
+from tsl.typing import DataArray, TemporalIndex, IndexSlice, TensArray, \
+    SparseTensArray
 from .batch import Batch
-from .data import Data
 from .batch_map import BatchMap, BatchMapItem
+from .data import Data
 from .mixin import DataParsingMixin
 from .preprocessing.scalers import Scaler, ScalerModule
 from .utils import SynchMode, WINDOW, HORIZON, broadcast, outer_pattern
@@ -28,21 +29,25 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
     build a proper structure to feed deep models.
 
     Args:
-        data (TensArray): Data relative to the primary channels.
+        data (DataArray): Data relative to the primary channels.
         index (TemporalIndex, optional): Temporal indices for the data.
             (default: :obj:`None`)
-        mask (TensArray, optional): Boolean mask denoting if signal in data is
+        mask (DataArray, optional): Boolean mask denoting if signal in data is
             valid (1) or not (0).
             (default: :obj:`None`)
-        connectivity (TensArray, tuple, optional): The adjacency matrix defining nodes'
-            relational information. It can be either a dense matrix
-            :math:`\mathbf{A} \in \mathbb{R}^{N \times N}` or in COO format as a
-            tuple (:obj:`edge_index` :math:`\in \mathbb{N}^{2 \times E}`,
-            :obj:`edge_weight` :math:`\in \mathbb{R}^{E})`.
+        connectivity (SparseTensArray, tuple, optional): The adjacency matrix
+            defining nodes' relational information. It can be either a
+            dense/sparse matrix :math:`\mathbf{A} \in \mathbb{R}^{N \times N}`
+            or an (:obj:`edge_index` :math:`\in \mathbb{N}^{2 \times E}`,
+            :obj:`edge_weight` :math:`\in \mathbb{R}^{E})` tuple. The input
+            layout will be preserved (e.g., a sparse matrix will be stored as a
+            :class:`torch_sparse.SparseTensor`). In any case, the connectivity
+            will be stored in the attribute :obj:`edge_index`, and the weights
+            will be eventually stored as :obj:`edge_weight`.
             (default: :obj:`None`)
         exogenous (dict, optional): Dictionary of exogenous channels with label.
             An :obj:`exogenous` element is a temporal array with node- or graph-
-            level channels which are covariate to the main signal. The temporal
+            level channels which are covariates to the main signal. The temporal
             dimension must be equal to the temporal dimension of data, as well
             as the number of nodes if the exogenous is node-level.
             (default: :obj:`None`)
@@ -56,7 +61,12 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
             mapping are keys in :obj:`item.input`, while values are
             :obj:`~tsl.data.new.BatchMapItem`.
             (default: :obj:`None`)
-        trend (TensArray, optional): Trend paired with main signal. Must be of
+        target_map (BatchMap or dict, optional): Defines how data, exogenous and
+            attributes are mapped to the target of dataset samples. Keys in the
+            mapping are keys in :obj:`item.target`, while values are
+            :obj:`~tsl.data.new.BatchMapItem`.
+            (default: :obj:`None`)
+        trend (DataArray, optional): Trend paired with main signal. Must be of
             the same shape of `data`.
             (default: :obj:`None`)
         scalers (Mapping or None): Dictionary of scalers that must be used for
@@ -74,15 +84,16 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
             horizon.
     """
 
-    def __init__(self, data: TensArray,
+    def __init__(self, data: DataArray,
                  index: Optional[TemporalIndex] = None,
-                 mask: Optional[TensArray] = None,
+                 mask: Optional[DataArray] = None,
                  connectivity: Optional[
-                     Union[TensArray, Tuple[TensArray]]] = None,
-                 exogenous: Optional[Mapping[str, TensArray]] = None,
-                 attributes: Optional[Mapping[str, TensArray]] = None,
+                     Union[SparseTensArray, Tuple[DataArray]]] = None,
+                 exogenous: Optional[Mapping[str, DataArray]] = None,
+                 attributes: Optional[Mapping[str, DataArray]] = None,
                  input_map: Optional[Union[Mapping, BatchMap]] = None,
-                 trend: Optional[TensArray] = None,
+                 target_map: Optional[Union[Mapping, BatchMap]] = None,
+                 trend: Optional[DataArray] = None,
                  scalers: Optional[Mapping[str, Scaler]] = None,
                  window: int = 24,
                  horizon: int = 24,
@@ -99,10 +110,17 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
         # Initialize private data holders
         self._exogenous = dict()
         self._attributes = dict()
+        self._indices = None
         self.input_map = BatchMap()
+        self.target_map = BatchMap()
         # Store data
         self.data: Tensor = self._parse_data(data)
         # Store time information
+        # if index is not explicitly passed and data is a dataframe, defaults to
+        # data's index
+        if index is None and isinstance(data, pd.DataFrame):
+            if isinstance(data.index, TemporalIndex.__args__):
+                index = data.index
         self.index: Optional[TemporalIndex] = index
         # Store mask
         self.mask: Optional[Tensor] = self._parse_mask(mask)
@@ -115,18 +133,29 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
         self.stride = stride
         self.window_lag = window_lag
         self.horizon_lag = horizon_lag
-        # Updated input map (i.e., how to map data, exogenous and attribute
-        # inside item)
-        if input_map is None:
-            input_map = self.default_input_map()
-        self.set_input_map(input_map)
         # Store exogenous and attributes
         if exogenous is not None:
             for name, value in exogenous.items():
-                self.add_exogenous(name, **self._exog_value_to_kwargs(value))
+                self.add_exogenous(name, **self._exog_value_to_kwargs(value),
+                                   add_to_input_map=False)
         if attributes is not None:
             for name, value in attributes.items():
-                self.add_attribute(name, **self._attr_value_to_kwargs(value))
+                self.add_attribute(name, **self._attr_value_to_kwargs(value),
+                                   add_to_batch=False)
+        # Updated input map (i.e., how to map data, exogenous and attribute
+        # inside item.input)
+        if input_map is None:
+            input_map = self.default_input_map()
+        self.set_input_map(input_map)
+
+        # Updated target map (i.e., how to map data, exogenous and attribute
+        # inside item.target)
+        if target_map is None:
+            target_map = BatchMap(y=BatchMapItem('data', SynchMode.HORIZON,
+                                                 cat_dim=None, preprocess=False,
+                                                 n_channels=self.n_channels))
+        self.set_target_map(target_map)
+
         # Store preprocessing options
         self.trend = None
         if trend is not None:
@@ -176,9 +205,7 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
 
     @property
     def targets(self) -> BatchMap:
-        return BatchMap(y=BatchMapItem('data', SynchMode.HORIZON,
-                                       cat_dim=None, preprocess=False,
-                                       n_channels=self.n_channels))
+        return self.target_map
 
     def default_input_map(self) -> BatchMap:
         im = BatchMap(x=BatchMapItem('data', SynchMode.WINDOW, cat_dim=None,
@@ -191,28 +218,37 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
         return im
 
     def set_input_map(self, input_map=None, **kwargs):
-        if input_map is None:
-            self.input_map = BatchMap()
-        elif isinstance(input_map, BatchMap):
-            self.input_map = input_map
-        elif isinstance(input_map, Mapping):
-            self.input_map = BatchMap(**input_map)
-        else:
-            raise TypeError(f"Type {type(input_map)} is not valid "
-                            f"for `input_map`")
-        self.update_input_map(**kwargs)
+        self.input_map = BatchMap()
+        self.update_input_map(input_map, **kwargs)
+
+    def set_target_map(self, target_map=None, **kwargs):
+        self.target_map = BatchMap()
+        self.update_target_map(target_map, **kwargs)
 
     def update_input_map(self, input_map=None, **kwargs):
-        keys = []
-        if input_map is not None:
-            self.input_map.update(**input_map)
-            keys += list(input_map.keys())
-        self.input_map.update(**kwargs)
-        keys += list(kwargs.keys())
-        for key, item in self.input_map.items():
-            if key in keys:
-                item.n_channels = sum([getattr(self, k).shape[-1]
-                                       for k in item.keys])
+        self._update_batch_map('input', input_map, **kwargs)
+
+    def update_target_map(self, target_map=None, **kwargs):
+        self._update_batch_map('target', target_map, **kwargs)
+
+    def _update_batch_map(self, endpoint, batch_map=None, **kwargs):
+        # check batch map
+        if batch_map is not None:
+            assert isinstance(batch_map, Mapping), \
+                f"Type {type(batch_map)} is not valid for `input_map`"
+        keys = set(kwargs.keys())  # store the updated keys to add 'n_channels'
+        # update from batch_map
+        endpoint = getattr(self, f"{endpoint}_map")
+        if batch_map is not None:
+            endpoint.update(**batch_map)
+            keys.update(batch_map.keys())
+        # update from kwargs
+        endpoint.update(**kwargs)
+        # update 'n_channels' to newly added/updated keys
+        for key in keys:
+            item = endpoint[key]
+            item.n_channels = sum([getattr(self, k).shape[-1]
+                                   for k in item.keys])
 
     @property
     def keys(self) -> list:
@@ -276,42 +312,110 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
 
     # Setters #################################################################
 
-    def set_data(self, data: TensArray):
+    def set_data(self, data: DataArray):
+        r"""Set sequence of primary channels at :obj:`self.data`."""
         self.data = self._parse_data(data)
 
-    def set_mask(self, mask: TensArray):
+    def set_mask(self, mask: DataArray):
+        r"""Set mask of primary channels, i.e., a bool for each (node, step,
+        channel) triple denoting if corresponding value in data is observed (1)
+        or not (0)."""
         self.mask = self._parse_mask(mask)
 
-    def set_connectivity(self, connectivity: Union[TensArray,
-                                                   Tuple[TensArray]]):
-        self.edge_index, self.edge_weight = self._parse_adj(connectivity)
+    def set_connectivity(self,
+                         connectivity: Union[SparseTensArray, Tuple[DataArray]],
+                         target_layout: Optional[str] = None):
+        r"""Set dataset connectivity.
 
-    def set_trend(self, trend: TensArray):
-        self.trend: Tensor = self._parse_node_level_exogenous(trend, 'trend')
+        The input can be either a
+        dense/sparse matrix :math:`\mathbf{A} \in \mathbb{R}^{N \times N}`
+        or an (:obj:`edge_index` :math:`\in \mathbb{N}^{2 \times E}`,
+        :obj:`edge_weight` :math:`\in \mathbb{R}^{E})` tuple. If
+        :obj:`target_layout` is :obj:`None`, the input layout will be
+        preserved (e.g., a sparse matrix will be stored as a
+        :class:`torch_sparse.SparseTensor`), otherwise the connectivity is
+        converted to the specified layout. In any case, the connectivity
+        will be stored in the attribute :obj:`edge_index`, and the weights
+        will be eventually stored as :obj:`edge_weight`.
+
+        Args:
+            connectivity (SparseTensArray, tuple, optional): The connectivity
+            target_layout (str, optional): If specified, the input connectivity
+                is converted to this layout. Possible options are [dense,
+                sparse, edge_index]. If :obj:`None`, the target layout is
+                inferred from the input.
+                (default: :obj:`None`)
+        """
+        self.edge_index, self.edge_weight = self._parse_adj(connectivity,
+                                                            target_layout)
+
+    def set_trend(self, trend: DataArray):
+        r"""Set trend of primary channels."""
+        self.trend: Tensor = self._parse_exogenous(trend, 'trend',
+                                                   node_level=True)
 
     def set_scaler(self, key: str, value: Scaler):
+        r"""Set a :class:`tsl.data.preprocessing.Scaler` for the temporal
+        variable :obj:`key`.
+
+        Args:
+            key (str): The name of the variable associated to the scaler. It
+                must be a temporal variable, i.e., :obj:`data` or an exogenous.
+            value (Scaler): The scaler.
+        """
         self.scalers[key] = value
 
     # Setter for secondary data
 
-    def add_exogenous(self, name: str, value: TensArray,
+    def add_exogenous(self, name: str, value: DataArray,
                       node_level: bool = True,
                       add_to_input_map: bool = True,
                       synch_mode: SynchMode = WINDOW,
                       preprocess: bool = True) -> "SpatioTemporalDataset":
+        r"""Add an exogenous variable to the dataset.
+
+        Exogenous variables are covariates to the main signal (stored in
+        :obj:`data`). They can either be graph-level (i.e., with same temporal
+        length as :obj:`data` but with no node dimension) or node-level (i.e.,
+        with same temporal and node size as :obj:`data`). The exogenous variable
+        can then be accessed as :obj:`dataset.{name}` or
+        :obj:`dataset.exogenous[{name}]`.
+
+        Args:
+            name (str): The name of the exogenous variable. If the name starts
+                with "global_", the variable is assumed to be graph-level
+                (overriding parameter :obj:`node_level`), and the "global_"
+                prefix is removed from the name.
+            value (DataArray): The data sequence. Can be a
+                :class:`~pandas.DataFrame`, a :class:`~numpy.ndarray` or a
+                :class:`~torch.Tensor`.
+            node_level (bool): Whether the input variable is node- or
+                graph-level. If a 2-dimensional array is given and node-level is
+                :obj:`True`, it is assumed that the input has one channel.
+                (default: :obj:`True`)
+            add_to_input_map (bool): Whether to map the exogenous variable to
+                dataset item when calling :obj:`get` methods.
+                (default: :obj:`True`)
+            synch_mode (SynchMode): How to synchronize the exogenous variable
+                inside dataset item, i.e., with the window slice
+                (:obj:`SynchMode.WINDOW`) or horizon (:obj:`SynchMode.HORIZON`).
+                (default: :obj:`SynchMode.WINDOW`)
+            preprocess (bool): If :obj:`True` and the dataset has a scaler with
+                same key, then data are scaled when calling :obj:`get` methods.
+                (default: :obj:`True`)
+
+        Returns:
+            SpatioTemporalDataset: the dataset with added exogenous.
+        """
         if name.startswith('global_'):
             name = name[7:]
             node_level = False
         # validate name
         self._check_name(name)
         # check if node-level or graph-level
-        if node_level:
-            value = self._parse_node_level_exogenous(value, name)
-        else:
-            value = self._parse_graph_level_exogenous(value, name)
+        value = self._parse_exogenous(value, name, node_level)
         # add exogenous and set as attribute
-        self._exogenous[name] = dict(value=value,
-                                     node_level=node_level)
+        self._exogenous[name] = dict(value=value, node_level=node_level)
         setattr(self, name, value)
         if add_to_input_map:
             im = {name: BatchMapItem(name, synch_mode, preprocess,
@@ -320,61 +424,112 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
         return self
 
     def update_exogenous(self, name: str,
-                         value: Optional[TensArray] = None,
+                         value: Optional[DataArray] = None,
                          node_level: Optional[bool] = None):
+        r"""Update an existing exogenous variable in the dataset.
+
+        Use this method if you want to update the exogenous data (:obj:`value`),
+        and/or the :obj:`node_level` attribute of the exogenous.
+
+        Args:
+            name (str): The name of the exogenous variable. There must exist an
+                exogenous with this name in the dataset.
+            value (DataArray, optional): The new data sequence. Can be a
+                :class:`~pandas.DataFrame`, a :class:`~numpy.ndarray` or a
+                :class:`~torch.Tensor`.
+                (default: :obj:`None`)
+            node_level (bool, optional): Whether the exogenous is node- or
+                graph-level.
+                (default: :obj:`None`)
+        """
         if name not in self._exogenous:
             raise AttributeError(f"No exogenous named '{name}'.")
-        # defaults to current value if None
+        # defaults to current values
         if value is None:
             value = self._exogenous[name]['value']
-        # if node_level is provided, parse value according to node_level
-        if node_level is True:
-            value = self._parse_node_level_exogenous(value, name)
-            self._exogenous[name]['node_level'] = node_level
-        elif node_level is False:
-            value = self._parse_graph_level_exogenous(value, name)
-            self._exogenous[name]['node_level'] = node_level
-        # update value (eventually)
+        if node_level is None:
+            node_level = self._exogenous[name]['node_level']
+        # parse value according to node_level
+        value = self._parse_exogenous(value, name, node_level)
+        # update exogenous
         self._exogenous[name]['value'] = value
+        self._exogenous[name]['node_level'] = node_level
         setattr(self, name, value)
 
-    def add_attribute(self, name: str, value: TensArray,
+    def add_attribute(self, name: str, value: DataArray,
                       node_level: bool = True,
                       add_to_batch: bool = True) -> "SpatioTemporalDataset":
+        r"""Add a static attribute to the dataset.
+
+        Attributes are static features related to the dataset. They can either
+        be graph-level (i.e., with no node dimension) or node-level (i.e.,
+        with same node size as :obj:`data`). Once added, an attribute can be
+        accessed as :obj:`dataset.{name}` or :obj:`dataset.attributes[{name}]`.
+
+        Args:
+            name (str): The name of the attribute. If the name starts
+                with "global_", the variable is assumed to be graph-level
+                (overriding parameter :obj:`node_level`), and the "global_"
+                prefix is removed from the name.
+            value (DataArray): The data sequence. Can be a
+                :class:`~pandas.DataFrame`, a :class:`~numpy.ndarray` or a
+                :class:`~torch.Tensor`.
+            node_level (bool): Whether the input variable is node- or
+                graph-level. If a 1-dimensional array is given and node-level is
+                :obj:`True`, it is assumed that the input has one channel.
+                (default: :obj:`True`)
+            add_to_batch (bool): Whether to map the attribute to dataset item
+                when calling :obj:`get` methods.
+                (default: :obj:`True`)
+        """
         if name.startswith('global_'):
             name = name[7:]
             node_level = False
         # validate name
         self._check_name(name)
-        if node_level:
-            value = self._parse_node_level_attribute(value, name)
-        else:
-            value = self._parse_graph_level_attribute(value)
+        value = self._parse_attribute(value, name, node_level)
         self._attributes[name] = dict(value=value, node_level=node_level,
                                       add_to_batch=add_to_batch)
         setattr(self, name, value)
         return self
 
     def update_attribute(self, name: str,
-                         value: Optional[TensArray] = None,
+                         value: Optional[DataArray] = None,
                          node_level: Optional[bool] = None,
                          add_to_batch: Optional[bool] = None):
+        r"""Update an existing attribute in the dataset.
+
+        Use this method if you want to update the attribute data (:obj:`value`),
+        and/or the level of the attribute (:obj:`node_level`).
+
+        Args:
+            name (str): The name of the attribute variable. There must exist an
+                attribute with this name in the dataset.
+            value (DataArray, optional): The new data. Can be a
+                :class:`~pandas.DataFrame`, a :class:`~numpy.ndarray` or a
+                :class:`~torch.Tensor`.
+                (default: :obj:`None`)
+            node_level (bool, optional): Whether the attribute is node- or
+                graph-level.
+                (default: :obj:`None`)
+            add_to_batch (bool): Whether to map the attribute to dataset item
+                when calling :obj:`get` methods.
+                (default: :obj:`None`)
+        """
         if name not in self._attributes:
             raise AttributeError(f"No attribute named '{name}'.")
-        # defaults to current value if None
+        # defaults to current values
         if value is None:
             value = self._attributes[name]['value']
-        # if node_level is provided, parse value according to node_level
-        if node_level is True:
-            value = self._parse_node_level_attribute(value, name)
-            self._attributes[name]['node_level'] = node_level
-        elif node_level is False:
-            value = self._parse_graph_level_attribute(value)
-            self._attributes[name]['node_level'] = node_level
+        if node_level is None:
+            node_level = self._attributes[name]['node_level']
+        # parse value according to node_level
+        value = self._parse_attribute(value, name, node_level)
         if add_to_batch is not None:
             self._attributes[name]['add_to_batch'] = add_to_batch
-        # update value (eventually)
+        # update attribute
         self._attributes[name]['value'] = value
+        self._attributes[name]['node_level'] = node_level
         setattr(self, name, value)
 
     # Dataset properties ######################################################
@@ -383,41 +538,62 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
 
     @property
     def horizon_offset(self) -> int:
+        """Lag of starting step of horizon."""
         return self.window + self.delay
 
     @property
     def sample_span(self) -> int:
+        """Total number of steps of an item, including window and horizon."""
         return max(self.horizon_offset + self.horizon, self.window)
 
     @property
     def samples_offset(self) -> int:
+        """Difference (in number of steps) between two items."""
         return int(np.ceil(self.window / self.stride))
 
     @property
     def indices(self) -> Tensor:
+        """Indices of the dataset. The :obj:`i`-th item is mapped to
+        :obj:`indices[i]`"""
         return self._indices
 
     # Shape
 
     @property
     def n_steps(self) -> int:
+        """Total number of time steps in the dataset."""
         return self.data.shape[0]
 
     @property
     def n_nodes(self) -> int:
+        """Number of nodes in the dataset."""
         return self.data.shape[1]
 
     @property
     def n_channels(self) -> int:
+        """Number of primary channels in the dataset."""
         return self.data.shape[-1]
 
     @property
     def n_edges(self) -> int:
+        """Number of edges in the dataset, if a connectivity is set."""
         return self.edge_index.size(1) if self.edge_index is not None else None
 
     @property
     def patterns(self):
+        """Shows the dimension of dataset's tensors in a more informative way.
+
+        The pattern mapping can be useful to glimpse on how data are arranged.
+        The convention we use is the following:
+          * 'b' stands for “batch size”
+          * 'c' stands for “number of channels” (per node)
+          * 'e' stands for “number edges”
+          * 'n' stands for “number of nodes”
+          * 's' stands for “number of time steps”
+        """
         patterns = dict(data='s n c')
+        if self.mask is not None:
+            patterns['mask'] = 's n c'
         patterns.update({key: 's n c' if value['node_level'] else 's c'
                          for key, value in self._exogenous.items()})
         patterns.update({key: 'n c' if value['node_level'] else 'c'
@@ -434,10 +610,12 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
 
     @property
     def exogenous(self) -> dict:
+        """Covariates to the main signal."""
         return {k: v['value'] for k, v in self._exogenous.items()}
 
     @property
     def attributes(self) -> dict:
+        """Static features related to the dataset."""
         return {k: v['value'] for k, v in self._attributes.items()}
 
     # Methods for accessing tensor ############################################
@@ -511,7 +689,7 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
         if self.index is None:
             return None
         ds_indices = self.expand_indices(indices, unique=unique)
-        index = self.index.to_numpy()
+        index = self.index if unique else self.index.to_numpy()
         ds_timestamps = {k: index[v] for k, v in ds_indices.items()}
         return ds_timestamps
 
@@ -519,10 +697,38 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
 
     def reduce(self, step_index: Optional[IndexSlice] = None,
                node_index: Optional[IndexSlice] = None):
+        """Reduce the dataset in terms of number of steps and nodes. Returns a
+        copy of the reduced dataset.
+
+        If dataset has a connectivity, edges ending to or starting from removed
+        nodes will be removed as well.
+
+        Args:
+            step_index (IndexSlice, optional): index or mask of the nodes to
+                keep after reduction.
+                (default: :obj:`None`)
+            node_index (IndexSlice, optional): index or mask of the nodes to
+                keep after reduction.
+                (default: :obj:`None`)
+        """
         return deepcopy(self).reduce_(step_index, node_index)
 
     def reduce_(self, step_index: Optional[IndexSlice] = None,
                 node_index: Optional[IndexSlice] = None):
+        """Reduce the dataset in terms of number of steps and nodes. This is an
+        inplace operation.
+
+        If dataset has a connectivity, edges ending to or starting from removed
+        nodes will be removed as well.
+
+        Args:
+            step_index (IndexSlice, optional): index or mask of the nodes to
+                keep after reduction.
+                (default: :obj:`None`)
+            node_index (IndexSlice, optional): index or mask of the nodes to
+                keep after reduction.
+                (default: :obj:`None`)
+        """
         if step_index is None:
             step_index = slice(None)
         if node_index is None:
@@ -586,16 +792,18 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
             return self.get_horizon_indices(item)
         return self.get_window_indices(item), self.get_horizon_indices(item)
 
+    def set_indices(self, indices: TensArray):
+        indices = torch.as_tensor(indices, dtype=torch.long)
+        max_index = self.n_steps - self.sample_span
+        assert all((indices >= 0) & (indices <= max_index)), \
+            f"indices must be in the range [0, {max_index}] for {self.name}."
+        self._indices = indices
+
     def expand_indices(self, indices=None, unique=False, merge=False):
         indices = np.arange(len(self._indices)) if indices is None else indices
         hrz_end = self.horizon_offset + self.horizon
 
         def expand_indices_range(rng_start, rng_end):
-            allowable_offset = rng_end - rng_start - self.stride + 1
-            contiguous = all(np.diff(indices) <= allowable_offset)
-            if unique and contiguous:
-                return np.arange(self._indices[indices[0]] + rng_start,
-                                 self._indices[indices[-1]] + rng_end)
             ind_mtrx = [self._indices[indices].numpy() + inc for inc in
                         range(rng_start, rng_end)]
             idx = np.swapaxes(ind_mtrx, 0, 1)
@@ -603,14 +811,13 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
 
         if merge:
             unique = True
-            return expand_indices_range(0, hrz_end)
+            start = 0 if self.window > 0 else self.horizon_offset
+            return expand_indices_range(start, hrz_end)
 
         ds_indices = dict()
         if self.window > 0:
             ds_indices[WINDOW] = expand_indices_range(0, self.window)
-        if self.horizon > 0:
-            ds_indices[HORIZON] = expand_indices_range(self.horizon_offset,
-                                                       hrz_end)
+        ds_indices[HORIZON] = expand_indices_range(self.horizon_offset, hrz_end)
         return ds_indices
 
     def overlapping_indices(self, idxs1, idxs2,
@@ -635,7 +842,7 @@ class SpatioTemporalDataset(Dataset, DataParsingMixin):
         return Batch.from_data_list([self.get(idx) for idx in indices])
 
     def numpy(self):
-        return np.as_array(self.data)
+        return np.asarray(self.data)
 
     def dataframe(self):
         columns = pd.MultiIndex.from_product([np.arange(self.n_nodes),
