@@ -2,17 +2,22 @@
 from .prototypes import Dataset, PandasDataset
 # Datasets
 from .air_quality import AirQuality
-from .electricity import Electricity
+from .elergone import Elergone
 from .metr_la import MetrLA
 from .pems_bay import PemsBay
+from .mts_benchmarks import ElectricityBenchmark, TrafficBenchmark, SolarBenchmark, ExchangeBenchmark
 
 __all__ = [
     'Dataset',
     'PandasDataset',
     'AirQuality',
-    'Electricity',
+    'Elergone',
     'MetrLA',
-    'PemsBay'
+    'PemsBay',
+    'ElectricityBenchmark',
+    'TrafficBenchmark',
+    'SolarBenchmark',
+    'ExchangeBenchmark'
 ]
 
 prototype_classes = __all__[:2]
