@@ -1,18 +1,21 @@
 from .dense_spatial_conv import SpatialConv, SpatialConvOrderK
 from .diff_conv import DiffConv
 from .graph_attention import AttentionScores, MultiHeadGraphAttention, GATLayer
+from .gat_conv import GATConv
 from .grin_cell import GRIL
 from .spatio_temporal_att import SpatioTemporalAtt
+from .gated_gn import GatedGraphNetwork
 
 __all__ = [
     'SpatialConv',
     'SpatialConvOrderK',
     'DiffConv',
-    'AttentionScores',
     'MultiHeadGraphAttention',
+    'GATConv',
     'GATLayer',
     'GRIL',
     'SpatioTemporalAtt',
+    'GatedGraphNetwork'
 ]
 
 classes = __all__
